# Generated by roxygen2: do not edit by hand

export(check_installed)
export(copy_tinytex)
export(install_tinytex)
export(is_tinytex)
export(latexmk)
export(lualatex)
export(parse_install)
export(parse_packages)
export(pdflatex)
export(r_texmf)
export(reinstall_tinytex)
export(tinytex_root)
export(tl_pkgs)
export(tl_sizes)
export(tlmgr)
export(tlmgr_conf)
export(tlmgr_install)
export(tlmgr_path)
export(tlmgr_remove)
export(tlmgr_repo)
export(tlmgr_search)
export(tlmgr_update)
export(tlmgr_version)
export(uninstall_tinytex)
export(use_tinytex)
export(xelatex)
import(stats)
import(tools)
import(utils)
importFrom(xfun,dir_exists)
importFrom(xfun,download_file)
importFrom(xfun,grep_sub)
importFrom(xfun,in_dir)
importFrom(xfun,is_linux)
importFrom(xfun,is_macos)
importFrom(xfun,is_unix)
importFrom(xfun,is_windows)
importFrom(xfun,raw_string)
importFrom(xfun,with_ext)
